#include "stdafx.h"
#include "utils.h"

HICON utils::GetIconFromWindow(HWND hWnd, bool bBigIcon)
{
	HICON hIcon = NULL;

	if ( bBigIcon )
	{
		SendMessageTimeout(hWnd, WM_GETICON, ICON_BIG, 0, SMTO_ABORTIFHUNG, 1000, (LPDWORD) &hIcon);
		if (!hIcon)
			hIcon = (HICON) GetClassLong(hWnd, GCL_HICON);
		if (!hIcon)
			SendMessageTimeout(hWnd, WM_QUERYDRAGICON, 0, 0, SMTO_ABORTIFHUNG, 1000, (LPDWORD) &hIcon);
		if (!hIcon)
			SendMessageTimeout(hWnd, WM_GETICON, ICON_SMALL, 0, SMTO_ABORTIFHUNG, 1000, (LPDWORD) &hIcon);
		if (!hIcon)
			hIcon = (HICON) GetClassLong(hWnd, GCL_HICONSM);
	}
	else
	{
		SendMessageTimeout(hWnd, WM_GETICON, ICON_SMALL, 0, SMTO_ABORTIFHUNG, 1000, (LPDWORD) &hIcon);
		if (!hIcon)
			hIcon = (HICON) GetClassLong(hWnd, GCL_HICONSM);
		if (!hIcon)
			SendMessageTimeout(hWnd, WM_QUERYDRAGICON, 0, 0, SMTO_ABORTIFHUNG, 1000, (LPDWORD) &hIcon);
		if (!hIcon)
			SendMessageTimeout(hWnd, WM_GETICON, ICON_BIG, 0, SMTO_ABORTIFHUNG, 1000, (LPDWORD) &hIcon);
		if (!hIcon)
			hIcon = (HICON) GetClassLong(hWnd, GCL_HICON);
	}

	return hIcon;
}

bool utils::IsAppWindow(HWND hWnd)
{
	bool bResult = true;
	LONG nStyle;
	HWND hOwner;
	HWND hParent;

	// if it is a WS_CHILD or not WS_VISIBLE, fail it
	nStyle = GetWindowLong(hWnd, GWL_STYLE);
	if ((nStyle & WS_CHILD) || !(nStyle & WS_VISIBLE))
		bResult = false;
	else
	{
		// if the window is a WS_EX_TOOLWINDOW fail it
		nStyle = GetWindowLong(hWnd, GWL_EXSTYLE);
		if (nStyle & WS_EX_TOOLWINDOW)
			bResult = false;
		else
		{
			// If this has an owner, then only accept this window
			// if the partent is not accepted
			hOwner = GetWindow(hWnd, GW_OWNER);
			if (hOwner)
			{
				if (!IsAppWindow(hOwner))
					bResult = false;
				else
				{
					// if it has a parent, fail
					hParent = GetParent(hWnd);
					if (hParent)
						bResult = false;
					else
					{
						if (GetWindowLong(hWnd, GWL_USERDATA) == 0x49474541)
						{
							bResult = false;
						}
					}
				}
			}
		}
	}

	return bResult;
}

void utils::ErrorBox(const char *szMessage)
{
	MessageBox(NULL, szMessage, "LiteStep :: Wharf", MB_OK | MB_ICONERROR | MB_TOPMOST | MB_SETFOREGROUND);
}

static int utils::ParseIconLocation(LPSTR szIconFile)
{
	LPSTR pszComma = strrchr(szIconFile, ',');

	if (!pszComma)
		return -1;

	*pszComma = 0;
	return atoi(pszComma + 1);
}

HICON utils::GetIconFromPathEx(LPCSTR pszPath, int cxDesired, int cyDesired)
{
	char szIconFile[MAX_PATH];
	int nIconIndex = -1;
	LPCSTR pszExt;

    if (strlen(pszPath) > 0 && pszPath[0])
    {
        if (stricmp(pszPath, ".none") == 0)
        {
            return NULL;
        }
        
        strncpy(szIconFile, pszPath, MAX_PATH);
        nIconIndex = ParseIconLocation(szIconFile);
        
        pszExt = strrchr(szIconFile, '.');
            
        if (pszExt && stricmp(pszExt, ".ico") == 0)
        {
            return (HICON) LoadImage(NULL, szIconFile, IMAGE_ICON,
                cxDesired, cyDesired, LR_LOADFROMFILE);
        }
        else if (nIconIndex >= 0)
        {
            HICON hIcon;
                
            ExtractIconEx(szIconFile, nIconIndex,
                (cxDesired <= 24 ? NULL : &hIcon),
                (cxDesired <= 24 ? &hIcon : NULL), 1);
                
            return hIcon;
        }
        else
        {
            SHFILEINFO shfi;
            char pszFileName[MAX_PATH];
            LPCSTR pszFile = szIconFile;
            UINT uFlags = NULL;
                
            if (strchr(szIconFile, '*'))
            {
                PathStripPath(szIconFile);
                uFlags |= SHGFI_USEFILEATTRIBUTES;
            }
            else if (!PathFileExists(szIconFile))
            {
                FindExecutable(szIconFile, NULL, pszFileName);
                pszFile = pszFileName;
            }
                
            SHGetFileInfo(pszFile, 0, &shfi, sizeof(SHFILEINFO),
                SHGFI_ICON | (cxDesired <= 24 ? SHGFI_SMALLICON : 0) | uFlags);
            return shfi.hIcon;
        }
    }
    else
    {
        // get generic folder icon
        SHFILEINFO shfi;
        
        SHGetFileInfo("*", FILE_ATTRIBUTE_DIRECTORY, &shfi, sizeof(SHFILEINFO),
            SHGFI_USEFILEATTRIBUTES | SHGFI_ICON | (cxDesired <= 24 ? SHGFI_SMALLICON : 0));
        return shfi.hIcon;
    }
}


///////////////////////////////////////////////////////////////////////////////
//
// Determine may we use specified path to extract icon or not...
// It is ugly hack of LoadLSIcon() body from 0.24.7 RC1 core...
//
bool utils::isIconPath(const char *szFile)
{
	return
		!(lstrcmpi(szFile, ".none") == 0) &&
		(
			0 == strnicmp(szFile, ".extract=", 9) ||
			PathIsDirectory(szFile) ||
			PathMatchSpec(szFile, "*.ico")
		);
}